# -*- coding: utf-8 -*-

from odoo import models, fields, _, api
import logging

_logger = logging.getLogger(__name__)

class AccountInvoice(models.Model):
    _inherit = 'account.invoice'

    medical_invoice_id = fields.Many2one('medical.invoice', string='Medical invoice')

    @api.multi
    def write(self, vals):
        super(AccountInvoice, self).write(vals)
        for invoice in self:
            insurance_config = self.env['medical.invoice.config'].search([('insurance_id', '=', invoice.partner_id.id)])
            if invoice.state == 'paid':
                if insurance_config and insurance_config.is_capitated:
                    benefits = invoice.medical_invoice_id.medical_benefit_ids.filtered(lambda x:x.insurance_plan_id.insurance_id == invoice.partner_id)
                else:
                    benefits = invoice.invoice_line_ids.filtered(lambda x:x.benefit_id.invoice_id == self and x.benefit_id.state == 'invoiced').mapped('benefit_id')
                benefits.write({'state': 'charged'})

    @api.multi
    def button_reconcile_refund(self):
        aml_ids = self.move_id.line_ids.filtered(lambda r: not r.reconciled and r.account_id.internal_type in ('payable','receivable'))
        for aml_id in aml_ids:
            self.refund_invoice_id.assign_outstanding_credit(aml_id.id)

    @api.multi
    def action_charge_refund_invoices(self):
        context = dict()
        context.update(self.env.context)
        context.update({'create': False, 'order_by': ['create_date']})
        action = self.env.ref('account.action_invoice_tree').read()[0]
        action['domain'] = str([('refund_invoice_id', '=', self.id)])
        action['context'] = context
        #action['target'] = 'main'
        return action

    @api.multi
    def action_invoice_open(self):
        benefit_obj = self.env['medical.benefit']
        for invoice in self:
            insurance_config = self.env['medical.invoice.config'].search([('insurance_id', '=', invoice.partner_id.id)])
            if insurance_config and insurance_config.is_capitated and insurance_config.pre_invoice_medical_action:
                description_date = ' ' + '-'.join(str(invoice.date_invoice).split('-')[0:2][::-1])
                invoice_line = self.env['account.invoice.line'].create({
                    'name': _('Month') + description_date,
                    'account_id': insurance_config.product_id.categ_id.property_account_income_categ_id.id,
                    'product_id': insurance_config.product_id.id,
                    'price_unit': insurance_config.capitated_payment_amount,
                    'quantity': 1,
                })
                invoice.invoice_line_ids = [(6,0,invoice_line.ids)]
        res = super(AccountInvoice, self).action_invoice_open()
        for invoice in self:
            insurance_config = self.env['medical.invoice.config'].search([('insurance_id', '=', invoice.partner_id.id)])
            if invoice.state == 'open':
                if invoice.type == 'out_refund':
                    benefits = invoice.invoice_line_ids.mapped('benefit_id')
                    if benefits:
                        benefits.write({'state': 'for_reinvoicing','invoice_id': False})
                else:
                    if insurance_config.is_capitated:
                        benefits = invoice.medical_invoice_id.medical_benefit_ids.filtered(lambda x:x.insurance_plan_id.insurance_id == invoice.partner_id)
                        benefits.write({'state': 'invoiced','invoice_id': invoice.id})
                    else:
                        benefits = invoice.invoice_line_ids.mapped('benefit_id')
                        benefits.write({'state': 'invoiced','invoice_id': invoice.id})
        return res


class AccountInvoiceLine(models.Model):
    _inherit = 'account.invoice.line'

    benefit_id = fields.Many2one('medical.benefit', 'Benefit')
    descriptor_id = fields.Many2one('medical.benefit.descriptor','Descriptor')
    professional_id = fields.Many2one('res.partner','Professional')
    patient_id = fields.Many2one('res.partner',string='Patient',related='benefit_id.patient_id')
    benefit_date = fields.Date("Benefit Date",related="benefit_id.benefit_date")
