# -*- coding: utf-8 -*-

from odoo import models, fields, _, api
from odoo.exceptions import ValidationError


class MedicalInvoiceConfig(models.Model):
    _name = 'medical.invoice.config'
    _description = 'Medical Invoice Config'

    insurance_id = fields.Many2one(
        string='Medical insurance',
        comodel_name='res.partner',
        required=True,
        domain=[('partner_type', '=', 'insurance')]
    )
    code = fields.Char(string='Code', related='insurance_id.ref')
    filing_day = fields.Integer(string='Filing day', required=True)
    is_capitated = fields.Boolean('Is capitated?')
    pre_invoice_medical_action = fields.Boolean('Pre Invoice per Medical Action?')
    capitated_payment_amount = fields.Float('Payment amount')
    product_id = fields.Many2one('product.product', string="Service", required="True",
                                 domain="[('is_medical_service', '=', True)]")
    due_days = fields.Integer('Due days', default=30)

    _sql_constraints = [(
        'check_filing_date',
        'CHECK(filing_day >= 1 and filing_day <= 31)',
        _('Filing day not valid')
    ), (
        'check_due_days',
        'CHECK(due_days > 0)',
        _('Invoice due days must be greater than zero')
    )]

    @api.onchange('is_capitated')
    def onchange_is_capitated(self):
        if not self.is_capitated:
            self.pre_invoice_medical_action = False

    @api.multi
    def name_get(self):
        res = list()
        for rec in self:
            filing_day = str(rec.filing_day)
            if rec.filing_day < 10:
                filing_day = '0' + filing_day
            res.append((rec.id, f'{rec.insurance_id.name} - {filing_day}'))
        return res

    @api.constrains('is_capitated', 'capitated_payment_amount')
    def validate_payment_amount(self):
        for rec in self:
            if rec.is_capitated:
                if rec.capitated_payment_amount <= 0:
                    raise ValidationError(
                        _('The payment amount must be greater than zero')
                    )

