from odoo import models, _, api
import logging
_logger = logging.getLogger(__name__)

class ReportInvoiceDetail(models.AbstractModel):

    _name = 'report.medical_invoicing.report_invoice_detail_template'

    @api.model
    def _get_report_values(self, docids, data=None):
        data = data if data is not None else {}
        invoice = self.env['account.invoice'].browse(docids[0])
        mb_obj = self.env['medical.benefit']
        medical_invoice = invoice.medical_invoice_id
        partner = invoice.partner_id
        insurance_config = self.env['medical.invoice.config'].search([('insurance_id', '=', partner.insurance_plan_id.insurance_id.id)])
        benefits = []
        if insurance_config and insurance_config.is_capitated:
            if medical_invoice:
                benefits.extend(mb_obj.search([
                    ('insurance_plan_id', '=', partner.insurance_plan_id.id),
                    ('presentation_date', '>=', medical_invoice.start_date),
                    ('presentation_date', '<=', medical_invoice.end_date),
                    ('state', '!=', 'invalid')
                ]).ids)
            self._cr.execute(
                    """SELECT DISTINCT ON(rp.ref,mbd.name_os) CONCAT(rp.ref,' - ',rp.name) as prof, CONCAT(rpp.vat,' - ', rpp.name) as pat, mbd.name_os as desc_code, rp.id as prof_id, mbd.id as desc_id
                    FROM medical_benefit mb JOIN
                    medical_benefit_descriptor mbd ON mbd.id = mb.descriptor_id JOIN
                    res_partner rp ON mb.professional_id = rp.id JOIN
                    res_partner rpp ON mb.patient_id = rpp.id
                    WHERE mb.id IN %s
                    ORDER BY rp.ref DESC, mbd.name_os""", [tuple(benefits)]
            )
        else:
            self._cr.execute(
                    """SELECT DISTINCT ON(rp.ref,mbd.name_os) CONCAT(rp.ref,' - ',rp.name) as prof, CONCAT(rpp.vat,' - ', rpp.name) as pat, mbd.name_os as desc_code, rp.id as prof_id, mbd.id as desc_id
                    FROM account_invoice_line ail JOIN 
                    res_partner rp ON ail.professional_id = rp.id JOIN 
                    medical_benefit mb ON ail.benefit_id = mb.id JOIN
                    medical_benefit_descriptor mbd ON mbd.id = mb.descriptor_id JOIN
                    res_partner rpp ON rpp.id = mb.patient_id
                    WHERE ail.invoice_id = %s 
                    ORDER BY rp.ref DESC,mbd.name_os""", [invoice.id]
            )
        records = dict()
        professional = ""
        for l in self.env.cr.dictfetchall():
            if l['prof'] != professional:
                professional = l['prof']
                records[professional] = dict()
            descriptor = l['desc_code']
            records[professional][descriptor] = list()
            if insurance_config and insurance_config.is_capitated:
                benef = self.env['medical.benefit'].search([('id','in',benefits),('professional_id','=',l['prof_id']),('descriptor_id','=',l['desc_id'])])
                for mb in benef:
                    records[professional][descriptor].append({
                        'patient': l['pat'],
                        'price_unit': mb.price,
                        'price_total': mb.price,
                    })
            else:
                i_lines = self.env['account.invoice.line'].search([('invoice_id','=',invoice.id),('professional_id','=',l['prof_id']),('descriptor_id','=',l['desc_id'])])
                for line in i_lines:
                    records[professional][descriptor].append({
                        'patient': l['pat'],
                        'price_unit': line.price_unit,
                        'price_total': line.price_total,
                    })
        return {
            'doc_ids': docids,
            'doc_model': 'account.invoice',
            'records': records,
        }


