from odoo import models, _, api
import logging
_logger = logging.getLogger(__name__)

class ReportPreInvoice(models.AbstractModel):

    _name = 'report.medical_invoicing.report_pre_invoice_template'

    @api.model
    def _get_report_values(self, docids, data=None):
        invoice_id = docids[0]
        invoice = self.env['account.invoice'].browse(invoice_id)
        mb_obj = self.env['medical.benefit']
        medical_invoice = invoice.medical_invoice_id
        partner = invoice.partner_id
        insurance_config = self.env['medical.invoice.config'].search([('insurance_id', '=', partner.insurance_plan_id.insurance_id.id)])
        benefits = []
        if insurance_config and insurance_config.is_capitated:
            if medical_invoice:
                benefits.extend(mb_obj.search([
                    ('insurance_plan_id', '=', partner.insurance_plan_id.id),
                    ('presentation_date', '>=', medical_invoice.start_date),
                    ('presentation_date', '<=', medical_invoice.end_date),
                    ('state', '!=', 'invalid')
                ]).ids)
            self._cr.execute(
                    """SELECT rp.id,rp.id,rp.display_name, SUM(mb.quantity), SUM(mb.price)
                    FROM medical_benefit mb JOIN
                    res_partner rp ON mb.professional_id = rp.id
                    WHERE mb.id IN %s
                    GROUP BY rp.id,rp.display_name
                    ORDER BY rp.ref ASC""", [tuple(benefits)]
            )
        else:
            self._cr.execute(
                    """SELECT rp.display_name, SUM(ail.quantity), SUM(ail.price_total) 
                    FROM account_invoice_line ail JOIN 
                    res_partner rp ON ail.professional_id = rp.id JOIN 
                    medical_benefit mb ON ail.benefit_id = mb.id 
                    WHERE ail.invoice_id = %s 
                    GROUP BY rp.id,rp.display_name
                    ORDER BY rp.ref ASC""", [invoice_id]
            )
        records = []

        if insurance_config and insurance_config.is_capitated:
            for l in self._cr.fetchall():
                benef = mb_obj.search([('id','in',benefits),('professional_id','=',l[1])])
                total = 0
                for b in benef:
                    total += b.price
                value = {
                    'professional': l[2],
                    'benefit_count': l[3],
                    'price_total': total,
                }
                records.append(value)

        else:
            for l in self._cr.fetchall():
                value = {
                    'professional': l[0],
                    'benefit_count': l[1],
                    'price_total': l[2],
                }
                records.append(value)

        return {
            'doc_ids': docids,
            'doc_model': 'account.invoice',
            'doc': invoice,
            'records': records,
        }


