# -*- coding: utf-8 -*-
###############################################################################
#   Copyright (c) 2020 Eynes/E-mips (Marco Folco)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
###############################################################################

import logging

from odoo import models, api, fields, _
from odoo.exceptions import ValidationError, UserError
# from odoo.exceptions import except_orm
# from odoo.addons.decimal_precision import decimal_precision as dp
# from odoo.tools import DEFAULT_SERVER_DATE_FORMAT, \
#         DEFAULT_SERVER_DATETIME_FORMAT, float_compare


_logger = logging.getLogger(__name__)

class AccountInvoiceRefundLine(models.TransientModel):
    _name = 'account.invoice.refund.line'
    _description = 'Lines of refund wizard'

    invoice_line_id = fields.Many2one('account.invoice.line','Invoice lines')
    price = fields.Float('price')



class AccountInvoiceRefund(models.TransientModel):
    _inherit = "account.invoice.refund"

    professional_id = fields.Many2one('res.partner','Professional',domain=lambda x:x.get_domain_professional_id())
    descriptor_id = fields.Many2one('medical.benefit.descriptor','Descriptor',domain=lambda x:x.get_domain_descriptor_id())
    date_from = fields.Date('Date from')
    date_to = fields.Date('Date to')

    @api.multi
    def get_domain_professional_id(self):
        domain = []
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        professionals = invoice.invoice_line_ids.filtered(lambda x:x.benefit_id.state not in ['not_charged','charged','for_reinvoicing'] and x.benefit_id.invoice_id == invoice).mapped('professional_id')
        domain.append(('id','in',professionals.ids if professionals else []))
        return domain

    @api.multi
    def get_domain_descriptor_id(self):
        domain = []
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        descriptors = invoice.invoice_line_ids.filtered(lambda x:x.benefit_id.state not in ['not_charged','charged','for_reinvoicing'] and x.benefit_id.invoice_id == invoice).mapped('descriptor_id')
        domain.append(('id','in',descriptors.ids if descriptors else []))
        return domain


    @api.multi
    def button_remove_all_lines(self):
        self.write({'invoice_line_ids': [(5,0,0)]})
        return {
            'context': self.env.context,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'account.invoice.refund',
            'res_id': self.id,
            'view_id': False,
            'type': 'ir.actions.act_window',
            'target': 'new',
        }


    @api.multi
    def button_add_filtered_lines(self):
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        invoice_lines = invoice.invoice_line_ids.filtered(lambda x:x.benefit_id.state not in ['not_charged','charged','for_reinvoicing'] and x.benefit_id.invoice_id == invoice)
        if self.descriptor_id:
            invoice_lines = invoice_lines.filtered(lambda x:x.descriptor_id == self.descriptor_id)
        if self.professional_id:
            invoice_lines = invoice_lines.filtered(lambda x:x.professional_id == self.professional_id)
        if self.date_from:
            invoice_lines = invoice_lines.filtered(lambda x:x.benefit_date >= self.date_from)
        if self.date_to:
            invoice_lines = invoice_lines.filtered(lambda x:x.benefit_date <= self.date_to)

        for il in invoice_lines:
            self.write({'invoice_line_ids': [(4,il.id)]})
        self.descriptor_id = self.professional_id = self.date_from = self.date_to = False
        return {
            'context': self.env.context,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'account.invoice.refund',
            'res_id': self.id,
            'view_id': False,
            'type': 'ir.actions.act_window',
            'target': 'new',
        }


    @api.onchange('invoice_line_ids')
    def _get_invoice_line_domain(self):
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        invoice_lines = invoice.invoice_line_ids.filtered(lambda x:x.benefit_id.state not in ['not_charged','charged','for_reinvoicing'] and x.benefit_id.invoice_id == invoice)
        return {
            'domain':{
                'invoice_line_ids': [('id','in',invoice_lines.ids)],
            }
        }

    @api.multi
    def get_invoice_line_ids(self, invoice):
        line_ids = super(AccountInvoiceRefund, self).get_invoice_line_ids(invoice)
        for line in invoice.invoice_line_ids:
            line_ids.remove(line.id)
        return line_ids

    @api.multi
    def invoice_refund(self):
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        if self.filter_refund == 'modify':
            if len(self.invoice_line_ids) == 0:
                raise ValidationError(_("There must be at least one invoice line selected"))
        res = super(AccountInvoiceRefund, self).invoice_refund()
        if isinstance(res,dict):
            if 'res_id' in res and self.filter_refund == 'modify':
                refund_id = res['res_id'] if res['res_id'] != 0 else res['domain'][1][2][0] # Based on the invoice_refund of the base model
                invoice_refund = self.env['account.invoice'].browse(refund_id)
                invoice_refund.medical_invoice_id = invoice.medical_invoice_id
                invoice_refund.invoice_line_ids = [(5,0,0)]
                invoice_lines = self.invoice_line_ids
                invoice_lines = self.env['account.invoice'].with_context(mode='modify')._refund_cleanup_lines(invoice_lines)
                invoice_refund.invoice_line_ids = invoice_lines
                invoice_refund.compute_taxes()
        return res

    @api.model
    def default_get(self, fields_list):
        res = super(AccountInvoiceRefund, self).default_get(fields_list)
        res.update({
            'filter_refund': 'modify',
        })
        return res

